#include "bj.h"
int NmiFire;
unsigned char *BjGfx =NULL; unsigned int nBjGfxLen =0; // All the graphics
unsigned char *BjMap =NULL; unsigned int nBjMapLen =0; // Background tile maps
unsigned char *BjRom =NULL; unsigned int nBjRomLen =0; // Program Rom (as in rom)
unsigned char *BjRam =NULL; unsigned int nBjRamLen =0; // Program Rom (as in rom)
unsigned char tiles[1536+1024+50][8][8];
unsigned char *BjPalSrc=NULL; unsigned int nBjPalLen=0;
unsigned int *BjPalReal=NULL; 


// Z80 accesses
static unsigned char BjZIn(unsigned short /*addr*/)
{
    return 0;
}

static void BjZOut(unsigned short /*addr*/,unsigned char /*val*/)
{
}


unsigned char BjMemRead(unsigned short addr)
{
	unsigned char inputs=0;

	if (addr==0xb000) {
 		if (DrvJoy1[5])
 			inputs|=0x01;
 		if (DrvJoy1[4])
 			inputs|=0x02;
 		if (DrvJoy1[2])
 			inputs|=0x04;
 		if (DrvJoy1[3])
 			inputs|=0x08;
 		if (DrvJoy1[6])
 			inputs|=0x10;
 		return inputs;
 	}
 	if (addr==0xb002) {
         if (DrvJoy1[0])
 			inputs|=0x01;
         if (DrvJoy1[1])
 			inputs|=0x04;
 		return inputs;
 	}
	return BjRam[addr];
}

static void BjMemWrite(unsigned short addr,unsigned char val)
{
	if (addr >= 0x9000 && addr <=0x93ff) // videoram
	{
		BjRam[addr]=val;
	}
	if (addr >= 0x9400 && addr <=0x97ff) // colorram
	{
		BjRam[addr]=val;
	}
	if (addr >= 0x9c00 && addr <=0x9cff) // Pal ram
	{
		BjPalSrc[addr-0x9c00]=val;
	}
	if (addr ==0xb000) 
	{
		NmiFire=val;
	}	
	else
	{
		BjRam[addr]=val;
	}
}

int BjZInit()						
{
    // Init the z80
	ZetIn=BjZIn; 
	ZetOut=BjZOut;
	ZetRead=BjMemRead; 
	ZetWrite=BjMemWrite;
	ZetInit();
	ZetMapArea    (0x0000,0x7fff,0,BjRom+0x0000); // Direct Read from ROM
	ZetMapArea    (0x0000,0x7fff,2,BjRom+0x0000); // Direct Fetch from ROM
	ZetMapArea    (0xc000,0xdfff,0,BjRom+0x8000); // Direct Read from ROM
	ZetMapArea    (0xc000,0xdfff,2,BjRom+0x8000); // Direct Fetch from ROM
//	ZetMapArea    (0x0000,0xffff,1,BjRam+0); 
	// fill in the holes just in case, otherwise we might get a crash.
//	ZetMapArea    (0x8000,0xbfff,0,BjRam+0x8000); 
	ZetMapArea    (0x8000,0xbfff,2,BjRam+0x8000); 
//	ZetMapArea    (0xe000,0xffff,0,BjRam+0xe000); 
	ZetMapArea    (0xe000,0xffff,2,BjRam+0xe000); 
	ZetMemEnd();
    // remember to do ZetReset() in main driver
     return 0;
}

int BjZExit()
{
	ZetExit();
	ZetRead=NULL; ZetWrite=NULL;
	ZetIn=NULL; ZetOut=NULL;
	return 0;
}


static int DrvDoReset()
{
	ZetReset();
	return 0;
}

void DecodeTiles(int start, int num,int off1,int off2, int off3)
{

      int c,y,x,dat1,dat2,dat3,col;
      for (c=0;c<num;c++)
      {
            for (y=0;y<8;y++)
            {
            dat1=BjGfx[off1+(c*8)+y];
            dat2=BjGfx[off2+(c*8)+y];
            dat3=BjGfx[off3+(c*8)+y];
			for (x=0;x<8;x++)
                  {
 				  col=0;
    			  if (dat1&1)
                     col|=4;
                  if (dat2&1)
                     col|=2;
                  if (dat3&1)
                     col|=1;
                  tiles[c+start][7-y][7-x]=col;
                  dat1>>=1;
                  dat2>>=1;
                  dat3>>=1;
                  }
            }
      }
}



int BjInit()
{
// Allocate memory
	nBjGfxLen=0x15000;
	nBjMapLen=0x1000;
	nBjRomLen=(0x2000*5);
	nBjRamLen=0xffff;
    BjGfx=(unsigned char *)malloc(nBjGfxLen);
    if (BjGfx==NULL) return 1;
    memset(BjGfx,0x11,nBjGfxLen);

    BjMap=(unsigned char *)malloc(nBjMapLen);
    if (BjMap==NULL) return 1;
    memset(BjMap,0x22,nBjMapLen);

	BjRom  =(unsigned char *)malloc(nBjRomLen);
    if (BjRom==NULL) return 1;
    memset(BjRom,0x33,nBjRomLen);

	BjRam  =(unsigned char *)malloc(nBjRamLen);
    if (BjRam==NULL) return 1;
    memset(BjRam,0x44,nBjRamLen);

	nBjPalLen=0xFF*10;
	BjPalSrc=(unsigned char *)malloc(nBjPalLen);     
	if (BjPalSrc==NULL) return 1;
    memset(BjPalSrc,0x55,nBjPalLen);

	BjPalReal=(unsigned int *)malloc(nBjPalLen);     
	if (BjPalReal==NULL) return 1;
    memset(BjPalReal,0x66,nBjPalLen);

// Load roms
	for (int i =0; i<5 ; i++)
	{
		BurnLoadRom(BjRom+(0x2000*i),i,1); // load code roms
	}
	for (i=0;i<3;i++)
	{
		BurnLoadRom(BjGfx+(0x1000*i),i+5,1);
	}
	for (i=0;i<5;i++)
	{
		BurnLoadRom(BjGfx+0x3000+(0x2000*i),i+8,1);
	}

	BurnLoadRom(BjMap,14,1); // load Background tile maps

// Set memory access & Init
    BjZInit();
	DrvDoReset();
    DecodeTiles(0,512,0,0x1000,0x2000);
    DecodeTiles(512,1024,0x3000,0x5000,0x7000);
    DecodeTiles(1536,1024,0x9000,0x1100,0x1300);
// done

	return 0;
}

int BjExit()
{
	if (BjRom!=NULL) 
		free(BjRom);
	BjRom=NULL;
	
	if (BjGfx!=NULL) 
		free(BjGfx);
	BjGfx=NULL;

	if (BjMap!=NULL) 
		free(BjMap);
	BjMap=NULL;

	if (BjRam!=NULL) 
		free(BjRam);
	BjRam=NULL;
	if (BjPalSrc!=NULL) 
		free(BjPalSrc);
	BjPalSrc=NULL;

	if (BjPalReal!=NULL) 
		free(BjPalReal);
	BjPalReal=NULL;
	BjZExit();
	return 0;
}

//////
// Start of code based on YAAME 
inline static unsigned int CalcCol(unsigned short val, int i) 
{
 	unsigned int nColour = val;
 	int r, g, b;
 	r = (nColour & 0x000F) << 3;	// Red
 	g = (nColour & 0x00F0) >> 2;  	// Green
 	b = (nColour & 0x0F00) >> 7;	// Blue
 	return BurnHighCol(r, g, b, i);
}

static int CalcAll()
{
	int i; 
	unsigned short value;
	unsigned short *ps; 
	unsigned int *pp;
//	ps=(unsigned short *)BjPalSrc;
	pp=BjPalReal;
	for (i=0; i<128; i++,ps++,pp++)
	{
		value=0;
		value=BjPalSrc[i*2+1]<<6;
		value|=BjPalSrc[(i*2)+1];
		*pp=CalcCol(value,i);
	}
	return 0;
}

void BjDrawTile(int number, int startx, int starty)
{
	int x,y;
	int tempCol;
	unsigned char *temp;
	CalcAll();
	for (y=0;y<8;y++)
	{
		for (x=0;x<8;x++)
		{
			tempCol=tiles[number][x][y];
			temp=pBurnDraw + (nBurnPitch * y)+(nBurnBpp * x)+(nBurnPitch * starty)+(nBurnBpp * startx);
		//	if (tempCol!=0)
			PutPix(temp,BjPalReal[tempCol]);
		}
	}
}

void BjDrawTile16(int number, int startx, int starty)
{
	BjDrawTile(number, startx,starty);
	BjDrawTile(number-2, startx+8,starty);
	BjDrawTile(number-1, startx+8,starty+8);
	BjDrawTile(number+1, startx,starty+8);
}

static int BjDraw()

{

	int x,y;
	int c;
	int tile;
	int colour;
	int BgSel=BjRam[0x9e00];
	int pos=0x9000,pos2=0x9400,pos3=0x200*(BgSel&7),pos4;
	pos4=pos3+0x100;
	BurnClearScreen();
	if (BgSel&0x10)
	{
		for (x=15;x>-1;x--)
		{
			for (y=0;y<16;y++)
			{
				BjDrawTile16((BjMap[pos3]<<2)+1538,(x<<4),(y<<4));
				pos3++;
				pos4++;
			}
		}
	}
	for (x=0;x<32;x++)
	{
		for (y=0;y<32;y++)
		{
			if (BgSel&0x10)
			{
				if (BjRam[pos2]&0x10)
				BjDrawTile(BjRam[pos++]+256,(31-x)*8,y*8);
				else
				BjDrawTile(BjRam[pos++],(31-x)*8,y*8);
			}
			else
			{
				if (BjRam[pos2]&0x10)
				BjDrawTile(BjRam[pos++]+256,(31-x)<<3,y<<3);
				else
				BjDrawTile(BjRam[pos++],(31-x)<<3,y<<3);
			}
		}
	}
	pos=0x9820;
	for (c=0;c<24;c++)
	{
		x=BjRam[pos+2]-1;
		y=BjRam[pos+3];
		if (x>8&&y>8)
		{
			tile=BjRam[pos]&0x7F;
			colour=BjRam[pos+1]&0xF;
			if (!(BjRam[pos]&0x80))
			{
				BjDrawTile16((tile<<2)+514,x,y);
	
			}
			else
			{
				tile&=31;
				BjDrawTile((tile<<4)+1024,x+8+16,y);
				BjDrawTile((tile<<4)+1025,x+8+16,y+8);
				BjDrawTile((tile<<4)+1026,x+16,y);
				BjDrawTile((tile<<4)+1027,x+16,y+8);
				BjDrawTile((tile<<4)+1028,x+8+16,y+16);
				BjDrawTile((tile<<4)+1029,x+8+16,y+8+16);
				BjDrawTile((tile<<4)+1030,x+16,y+16);
				BjDrawTile((tile<<4)+1031,x+16,y+8+16);
				BjDrawTile((tile<<4)+1032,x+8,y);
				BjDrawTile((tile<<4)+1033,x+8,y+8);
				BjDrawTile((tile<<4)+1034,x,y);
				BjDrawTile((tile<<4)+1035,x,y+8);
				BjDrawTile((tile<<4)+1036,x+8,y+16);
				BjDrawTile((tile<<4)+1037,x+8,y+8+16);
				BjDrawTile((tile<<4)+1038,x,y+16);
				BjDrawTile((tile<<4)+1039,x,y+8+16);
			}
		}
		pos+=4;
	}
	return 0;
}
// end

int BjFrame()
{
	ZetRun(4000000/60);
	if (pBurnDraw != NULL) 
	{
		BjDraw();	// Draw screen if needed
	}
	if (NmiFire)
    	ZetNmi();
	return 0;
}

int BjScan(int nAction,int *pnMin)
{
	return 0;
}
